@@ -9,23 +9,29 @@ import json
 import os
 import re
 
-from tornado.web import HTTPError
+from tornado.web import HTTPError, RequestHandler
 
+from ...files.handlers import FilesHandler
 from .checkpoints import Checkpoints
 from traitlets.config.configurable import LoggingConfigurable
-from nbformat import sign, validate, ValidationError
+from nbformat import sign, validate as validate_nb, ValidationError
 from nbformat.v4 import new_notebook
 from ipython_genutils.importstring import import_item
 from traitlets import (
     Any,
+    Bool,
     Dict,
     Instance,
     List,
     TraitError,
     Type,
     Unicode,
+    validate,
+    default,
 )
 from ipython_genutils.py3compat import string_types
+from notebook.base.handlers import IPythonHandler
+
 
 copy_pat = re.compile(r'\-Copy\d*\.')
 
@@ -48,6 +54,10 @@ class ContentsManager(LoggingConfigurable):
       indicating the root path.
 
     """
+    
+    root_dir = Unicode('/', config=True)
+
+    allow_hidden = Bool(False, config=True, help="Allow access to hidden files")
 
     notary = Instance(sign.NotebookNotary)
     def _notary_default(self):
@@ -60,7 +70,7 @@ class ContentsManager(LoggingConfigurable):
         Glob patterns to hide in file and directory listings.
     """)
 
-    untitled_notebook = Unicode("Untitled", config=True,
+    untitled_notebook = Unicode(_("Untitled"), config=True,
         help="The base name used when creating untitled notebooks."
     )
 
@@ -91,12 +101,15 @@ class ContentsManager(LoggingConfigurable):
         - contents_manager: this ContentsManager instance
         """
     )
-    def _pre_save_hook_changed(self, name, old, new):
-        if new and isinstance(new, string_types):
-            self.pre_save_hook = import_item(self.pre_save_hook)
-        elif new:
-            if not callable(new):
-                raise TraitError("pre_save_hook must be callable")
+
+    @validate('pre_save_hook')
+    def _validate_pre_save_hook(self, proposal):
+        value = proposal['value']
+        if isinstance(value, string_types):
+            value = import_item(self.pre_save_hook)
+        if not callable(value):
+            raise TraitError("pre_save_hook must be callable")
+        return value
 
     def run_pre_save_hook(self, model, path, **kwargs):
         """Run the pre-save hook if defined, and log errors"""
@@ -111,15 +124,52 @@ class ContentsManager(LoggingConfigurable):
     checkpoints = Instance(Checkpoints, config=True)
     checkpoints_kwargs = Dict(config=True)
 
-    def _checkpoints_default(self):
+    @default('checkpoints')
+    def _default_checkpoints(self):
         return self.checkpoints_class(**self.checkpoints_kwargs)
 
-    def _checkpoints_kwargs_default(self):
+    @default('checkpoints_kwargs')
+    def _default_checkpoints_kwargs(self):
         return dict(
             parent=self,
             log=self.log,
         )
 
+    files_handler_class = Type(
+        FilesHandler, klass=RequestHandler, allow_none=True, config=True,
+        help="""handler class to use when serving raw file requests.
+
+        Default is a fallback that talks to the ContentsManager API,
+        which may be inefficient, especially for large files.
+
+        Local files-based ContentsManagers can use a StaticFileHandler subclass,
+        which will be much more efficient.
+
+        Access to these files should be Authenticated.
+        """
+    )
+
+    files_handler_params = Dict(
+        config=True,
+        help="""Extra parameters to pass to files_handler_class.
+
+        For example, StaticFileHandlers generally expect a `path` argument
+        specifying the root directory from which to serve files.
+        """
+    )
+
+    def get_extra_handlers(self):
+        """Return additional handlers
+
+        Default: self.files_handler_class on /files/.*
+        """
+        handlers = []
+        if self.files_handler_class:
+            handlers.append(
+                (r"/files/(.*)", self.files_handler_class, self.files_handler_params)
+            )
+        return handlers
+
     # ContentsManager API part 1: methods that must be
     # implemented in subclasses.
 
@@ -270,21 +320,26 @@ class ContentsManager(LoggingConfigurable):
             The name of a file, including extension
         path : unicode
             The API path of the target's directory
+        insert: unicode
+            The characters to insert after the base filename
 
         Returns
         -------
         name : unicode
             A filename that is unique, based on the input filename.
         """
+        # Extract the full suffix from the filename (e.g. .tar.gz)
         path = path.strip('/')
-        basename, ext = os.path.splitext(filename)
+        basename, dot, ext = filename.partition('.')
+        suffix = dot + ext
+
         for i in itertools.count():
             if i:
                 insert_i = '{}{}'.format(insert, i)
             else:
                 insert_i = ''
-            name = u'{basename}{insert}{ext}'.format(basename=basename,
-                insert=insert_i, ext=ext)
+            name = u'{basename}{insert}{suffix}'.format(basename=basename,
+                insert=insert_i, suffix=suffix)
             if not self.exists(u'{}/{}'.format(path, name)):
                 break
         return name
@@ -292,9 +347,9 @@ class ContentsManager(LoggingConfigurable):
     def validate_notebook_model(self, model):
         """Add failed-validation message to model"""
         try:
-            validate(model['content'])
+            validate_nb(model['content'])
         except ValidationError as e:
-            model['message'] = u'Notebook Validation failed: {}:\n{}'.format(
+            model['message'] = u'Notebook validation failed: {}:\n{}'.format(
                 e.message, json.dumps(e.instance, indent=1, default=lambda obj: '<UNKNOWN>'),
             )
         return model
@@ -411,9 +466,9 @@ class ContentsManager(LoggingConfigurable):
         """
         model = self.get(path)
         nb = model['content']
-        self.log.warn("Trusting notebook %s", path)
+        self.log.warning("Trusting notebook %s", path)
         self.notary.mark_cells(nb, True)
-        self.save(model, path)
+        self.check_and_sign(nb, path)
 
     def check_and_sign(self, nb, path=''):
         """Check for trusted cells, and sign the notebook.
@@ -430,7 +485,7 @@ class ContentsManager(LoggingConfigurable):
         if self.notary.check_cells(nb):
             self.notary.sign(nb)
         else:
-            self.log.warn("Saving untrusted notebook %s", path)
+            self.log.warning("Notebook %s is not trusted", path)
 
     def mark_trusted_cells(self, nb, path=''):
         """Mark cells as trusted if the notebook signature matches.
@@ -446,7 +501,7 @@ class ContentsManager(LoggingConfigurable):
         """
         trusted = self.notary.check_signature(nb)
         if not trusted:
-            self.log.warn("Notebook %s is not trusted", path)
+            self.log.warning("Notebook %s is not trusted", path)
         self.notary.mark_cells(nb, trusted)
 
     def should_list(self, name):
